@@ -8,7 +8,7 @@ use Text::Wrap   ();
 use Scalar::Util ();
 use version      ();
 
-our $VERSION = '0.19';
+our $VERSION = '0.21';
 
 # From DateTime::Format::W3CDTF
 our $W3CDTF_REGEX = qr{(\d\d\d\d) # Year
@@ -28,6 +28,11 @@ our $W3CDTF_REGEX = qr{(\d\d\d\d) # Year
 my @m = qw( Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec );
 my %months = map { $m[ $_ ] => $_ + 1 } 0 .. 11;
 
+our $UNKNOWN_VALS = join( '|', (
+    'Unknown Release Date', 'Unknown', 'Not Released', 'Development Release', 
+    'Development', 'Developer Release', 
+) );
+
 sub new {
     my $class = shift;
     return bless {
@@ -63,65 +68,80 @@ sub load_string {
         ? qr/^(?:$version::LAX|$changes->{next_token})/
         : qr/^$version::LAX/;
 
-    $preamble .= shift @lines while @lines && $lines[ 0 ] !~ $version_line_re;
+    $preamble .= shift( @lines ) . "\n" while @lines && $lines[ 0 ] !~ $version_line_re;
 
     for my $l ( @lines ) {
 
         # Version & Date
         if ( $l =~ $version_line_re ) {
-            my ( $v, $d ) = split m{\s+}, $l, 2;
-
-            # munge date formats, ignore junk
-            if ( $d ) {
+            my ( $v, $n ) = split m{\s+}, $l, 2;
+            my $match = '';
+            my $d;
 
+            # munge date formats, save the remainder as note
+            if ( $n ) {
+                # unknown dates
+                if ( $n =~ m{^($UNKNOWN_VALS)}i ) {
+                    $d     = $1;
+                    $match = $d;
+                }
                 # handle localtime-like timestamps
-                if ( $d
-                    =~ m{\D{3}\s+(\D{3})\s+(\d{1,2})\s+([\d:]+)?\D*(\d{4})} )
+                elsif ( $n
+                    =~ m{^(\D{3}\s+(\D{3})\s+(\d{1,2})\s+([\d:]+)?\D*(\d{4}))} )
                 {
-                    if ( $3 ) {
+                    $match = $1;
+                    if ( $4 ) {
 
                         # unfortunately ignores TZ data
                         $d = sprintf(
                             '%d-%02d-%02dT%sZ',
-                            $4, $changes->{ months }->{ $1 },
-                            $2, $3
+                            $5, $changes->{ months }->{ $2 },
+                            $3, $4
                         );
                     }
                     else {
                         $d = sprintf( '%d-%02d-%02d',
-                            $4, $changes->{ months }->{ $1 }, $2 );
+                            $5, $changes->{ months }->{ $2 }, $3 );
                     }
                 }
 
                 # RFC 2822
-                elsif ( $d
-                    =~ m{\D{3}, (\d{1,2}) (\D{3}) (\d{4}) (\d\d:\d\d:\d\d) ([+-])(\d{2})(\d{2})}
+                elsif ( $n
+                    =~ m{^(\D{3}, (\d{1,2}) (\D{3}) (\d{4}) (\d\d:\d\d:\d\d) ([+-])(\d{2})(\d{2}))}
                     )
                 {
+                    $match = $1;
                     $d = sprintf(
                         '%d-%02d-%02dT%s%s%02d:%02d',
-                        $3, $changes->{ months }->{ $2 },
-                        $1, $4, $5, $6, $7
+                        $4, $changes->{ months }->{ $3 },
+                        $2, $5, $6, $7, $8
                     );
                 }
 
                 # handle dist-zilla style, again ingoring TZ data
-                elsif ( $d
-                    =~ m{(\d{4}-\d\d-\d\d)\s+(\d\d:\d\d(?::\d\d)?)(\s+\D+)?} )
+                elsif ( $n
+                    =~ m{^((\d{4}-\d\d-\d\d)\s+(\d\d:\d\d(?::\d\d)?)(\s+\D+)?)} )
                 {
-                    $d = sprintf( '%sT%sZ', $1, $2 );
+                    $match = $1;
+                    $d = sprintf( '%sT%sZ', $2, $3 );
                 }
 
                 # start with W3CDTF, ignore rest
-                elsif ( $d =~ m{^($W3CDTF_REGEX)}p ) {
-                    $d = ${^MATCH};
+                elsif ( $n =~ m{^($W3CDTF_REGEX)}p ) {
+                    $d     = ${^MATCH};
+                    $match = $d;
                 }
+
+                # clean date from note
+                $n =~ s{^$match\s*}{};
             }
 
             push @releases,
                 CPAN::Changes::Release->new(
-                version => $v,
-                date    => $d,
+                version      => $v,
+                date         => $d,
+                _parsed_date => $match,
+                note         => $n,
                 );
             $ingroup = undef;
             $indent  = undef;
@@ -262,8 +282,11 @@ sub serialize {
     my $output;
 
     $output = $self->preamble . "\n\n" if $self->preamble;
-    $output .= join "\n", $_->serialize( %release_args )
-        for reverse $self->releases;
+
+    my @r = $self->releases;
+    @r = reverse @r unless $args{reverse};  # not a typo!
+
+    $output .= $_->serialize( %release_args ) for @r;
 
     return $output;
 }
@@ -351,7 +374,7 @@ may be either a regular hashref, or a L<CPAN::Changes::Release> object.
     
     # Release object argument
     my $rel = CPAN::Changes::Release->new(
-        version => '0.01', date => '2009-07-06
+        version => '0.01', date => '2009-07-06'
     );
     $changes->releases( $rel );
 
@@ -369,11 +392,14 @@ Deletes all of the releases specified by the versions supplied to the method.
 Returns the release object for the specified version. Should there be no 
 matching release object, undef is returned.
 
-=head2 serialize( group_sort => \&sorting_function )
+=head2 serialize( reverse => $boolean, group_sort => \&sorting_function )
 
 Returns all of the data as a string, suitable for saving as a Changes 
 file.
 
+If I<reverse> is provided and true, the releases are
+printed in the reverse order (oldest to latest).
+
 If I<group_sort> is provided, change groups are
 sorted according to the given function. If not,
 groups are sorted alphabetically.
@@ -437,7 +463,7 @@ Brian Cassidy E<lt>bricas@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2011 by Brian Cassidy
+Copyright 2011-2013 by Brian Cassidy
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
