@@ -5,9 +5,11 @@ use warnings;
 
 use CPAN::Changes;
 use Test::Builder;
+use version ();
 
-my $Test       = Test::Builder->new;
-my $version_re = '^[._\-[:alnum:]]+$';    # "Looks like" a version
+our $VERSION = '0.20';
+
+my $Test     = Test::Builder->new;
 
 sub import {
     my $self = shift;
@@ -27,8 +29,9 @@ sub changes_ok {
 }
 
 sub changes_file_ok {
-    my ( $file ) = @_;
+    my ( $file, $arg ) = @_;
     $file ||= 'Changes';
+    $arg ||= {};
 
     my $changes = eval { CPAN::Changes->load( $file ) };
 
@@ -50,12 +53,21 @@ sub changes_file_ok {
     $Test->ok( 1, "$file contains at least one release" );
 
     for ( @releases ) {
-        if ( $_->date !~ m{^${CPAN::Changes::W3CDTF_REGEX}\s*$} ) {
+        if ( !defined $_->date || $_->date eq ''  ) {
             $Test->ok( 0, "$file contains an invalid release date" );
-            $Test->diag( '  ERR: ' . $_->date );
+            $Test->diag( '  ERR: No date at version ' . $_->version );
             return;
         }
-        if ( $_->version !~ m{$version_re} ) {
+
+        my $d = $_->{ _parsed_date };
+        if ( $d !~ m[^${CPAN::Changes::W3CDTF_REGEX}$]
+                && $d !~ m[^(${CPAN::Changes::UNKNOWN_VALS})$] ) {
+            $Test->carp( 'Date "' . $d . '" is not in the recommend format' );
+        }
+
+        # strip off -TRIAL before testing
+        (my $version = $_->version) =~ s/-TRIAL$//;
+        if ( not version::is_lax($version) ) {
             $Test->ok( 0, "$file contains an invalid version number" );
             $Test->diag( '  ERR: ' . $_->version );
             return;
@@ -65,6 +77,26 @@ sub changes_file_ok {
     $Test->ok( 1, "$file contains valid release dates" );
     $Test->ok( 1, "$file contains valid version numbers" );
 
+    if ( defined $arg->{version} ) {
+        my $v = $arg->{version};
+
+        if ( my $release = $changes->release( $v ) ) {
+            $Test->ok( 1, "$file has an entry for the current version, $v" );
+            my $changes = $release->changes;
+
+            if ( $changes and grep { @$_ > 0 } values %$changes ) {
+              $Test->ok( 1, "entry for the current version, $v, has content" );
+            } else {
+              $Test->ok( 0, "entry for the current version, $v, no content" );
+            }
+        } else {
+            # Twice so that we have a fixed number of tests to plan.
+            # -- rjbs, 2011-05-02
+            $Test->ok( 0, "$file has no entry for the current version, $v" );
+            $Test->ok( 0, "$file has no entry for the current version, $v" );
+        }
+    }
+
     return $changes;
 }
 
@@ -95,10 +127,14 @@ changelogs match the specification.
 Simple wrapper around C<changes_file_ok>. Declares a four test plan, and 
 uses the default filename of C<Changes>.
 
-=head2 changes_file_ok( [ $file ] )
+=head2 changes_file_ok( $filename, \%arg )
 
 Checks the contents of the changes file against the specification. No plan 
-is declared and if no filename is specified, C<Changes> is used.
+is declared and if the filename is undefined, C<Changes> is used.
+
+C<%arg> may include a I<version> entry, in which case the entry for that
+version must exist and have content.  This is useful to ensure that the version
+currently being released has documented changes.
 
 =head1 SEE ALSO
 
@@ -116,7 +152,7 @@ Brian Cassidy E<lt>bricas@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2011 by Brian Cassidy
+Copyright 2011-2013 by Brian Cassidy
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
